@@ -1,7 +1,7 @@
 /**
  * @file dshow.cpp Windows DirectShow video-source
  *
- * Copyright (C) 2010 Creytiv.com
+ * Copyright (C) 2010 Alfred E. Heggestad
  * Copyright (C) 2010 Dusan Stevanovic
  */
 
@@ -97,8 +97,6 @@ DEFINE_GUID(CLSID_SampleGrabber, 0xc1f400a0, 0x3f08, 0x11d3,
 class Grabber;
 
 struct vidsrc_st {
-	const struct vidsrc *vs;  /* inheritance */
-
 	ICaptureGraphBuilder2 *capture;
 	IBaseFilter *grabber_filter;
 	IBaseFilter *dev_filter;
@@ -501,8 +499,6 @@ static int alloc(struct vidsrc_st **stp, const struct vidsrc *vs,
 	if (err)
 		goto out;
 
-	st->vs = vs;
-
 	st->size   = *size;
 	st->frameh = frameh;
 	st->arg    = arg;
