@@ -8,7 +8,8 @@
 #include <baresip.h>
 #include <gtk/gtk.h>
 #include "gtk_mod.h"
-
+#include <pthread.h>
+#include <string.h>
 
 struct call_window {
 	struct gtk_mod *mod;
@@ -46,6 +47,7 @@ enum call_window_events {
 	MQ_TRANSFER,
 };
 
+static pthread_mutex_t last_data_mut = PTHREAD_MUTEX_INITIALIZER;
 static struct call_window *last_call_win = NULL;
 static struct vumeter_dec *last_dec = NULL;
 static struct vumeter_enc *last_enc = NULL;
@@ -143,30 +145,44 @@ static void call_window_set_vu_enc(struct call_window *win,
 
 void call_window_got_vu_dec(struct vumeter_dec *dec)
 {
-	if (last_call_win)
+	pthread_mutex_lock(&last_data_mut);
+	if (last_call_win) {
 		call_window_set_vu_dec(last_call_win, dec);
+		last_dec = NULL;
+	}
 	else
 		last_dec = dec;
+	pthread_mutex_unlock(&last_data_mut);
 }
 
 
 void call_window_got_vu_enc(struct vumeter_enc *enc)
 {
-	if (last_call_win)
+	pthread_mutex_lock(&last_data_mut);
+	if (last_call_win) {
 		call_window_set_vu_enc(last_call_win, enc);
+		last_enc = NULL;
+	}
 	else
 		last_enc = enc;
+	pthread_mutex_unlock(&last_data_mut);
 }
 
 
 static void got_call_window(struct call_window *win)
 {
-	if (last_enc)
+	pthread_mutex_lock(&last_data_mut);
+	if (last_enc) {
 		call_window_set_vu_enc(win, last_enc);
-	if (last_dec)
+		last_enc = NULL;
+	}
+	if (last_dec) {
 		call_window_set_vu_dec(win, last_dec);
+		last_dec = NULL;
+	}
 	if (!last_enc || !last_dec)
 		last_call_win = win;
+	pthread_mutex_unlock(&last_data_mut);
 }
 
 
@@ -227,6 +243,7 @@ static gboolean call_on_key_press(GtkWidget *window, GdkEvent *ev,
 	case '1': case '2': case '3':
 	case '4': case '5': case '6':
 	case '7': case '8': case '9': case '0':
+	case 'a': case 'b': case 'c': case 'd':
 		re_snprintf(wavfile, sizeof wavfile, "sound%c.wav", key);
 		break;
 	case '*':
@@ -277,12 +294,15 @@ static void mqueue_handler(int id, void *data, void *arg)
 	switch ((enum call_window_events)id) {
 
 	case MQ_HANGUP:
-		ua_hangup(uag_current(), win->call, 0, NULL);
+		if (!win->closed) {
+			ua_hangup(call_get_ua(win->call), win->call, 0, NULL);
+			win->closed = true;
+		}
 		break;
 
 	case MQ_CLOSE:
 		if (!win->closed) {
-			ua_hangup(uag_current(), win->call, 0, NULL);
+			ua_hangup(call_get_ua(win->call), win->call, 0, NULL);
 			win->closed = true;
 		}
 		mem_deref(win);
@@ -323,8 +343,9 @@ static void call_window_destructor(void *arg)
 	if (window->vumeter_timer_tag)
 		g_source_remove(window->vumeter_timer_tag);
 
-	/* TODO: avoid race conditions here */
+	pthread_mutex_lock(&last_data_mut);
 	last_call_win = NULL;
+	pthread_mutex_unlock(&last_data_mut);
 }
 
 
@@ -349,7 +370,7 @@ struct call_window *call_window_new(struct call *call, struct gtk_mod *mod)
 	gtk_window_set_type_hint(GTK_WINDOW(window),
 			GDK_WINDOW_TYPE_HINT_DIALOG);
 
-	vbox = gtk_vbox_new(FALSE, 0);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	gtk_container_add(GTK_CONTAINER(window), vbox);
 
 	/* Peer name and URI */
@@ -368,7 +389,7 @@ struct call_window *call_window_new(struct call *call, struct gtk_mod *mod)
 	gtk_box_pack_start(GTK_BOX(vbox), status, FALSE, FALSE, 0);
 
 	/* Progress bars */
-	hbox = gtk_hbox_new(FALSE, 0);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	gtk_box_set_spacing(GTK_BOX(hbox), 6);
 	gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
@@ -390,7 +411,7 @@ struct call_window *call_window_new(struct call *call, struct gtk_mod *mod)
 	gtk_box_pack_end(GTK_BOX(hbox), image, FALSE, FALSE, 0);
 
 	/* Buttons */
-	button_box = gtk_hbutton_box_new();
+	button_box = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
 	gtk_button_box_set_layout(GTK_BUTTON_BOX(button_box),
 			GTK_BUTTONBOX_END);
 	gtk_box_set_spacing(GTK_BOX(button_box), 6);
@@ -477,6 +498,7 @@ void call_window_closed(struct call_window *win, const char *reason)
 {
 	char buf[256];
 	const char *status;
+	const char *user_trigger_reason = "Connection reset by user";
 
 	if (!win)
 		return;
@@ -501,6 +523,12 @@ void call_window_closed(struct call_window *win, const char *reason)
 	call_window_set_status(win, status);
 	win->transfer_dialog = mem_deref(win->transfer_dialog);
 	win->closed = true;
+
+	if (reason && strncmp(reason, user_trigger_reason,
+	    strlen(user_trigger_reason)) == 0) {
+		mqueue_push(win->mq, MQ_CLOSE, win);
+		return;
+	}
 }
 
 
@@ -516,7 +544,9 @@ void call_window_progress(struct call_window *win)
 		return;
 
 	win->duration_timer_tag = g_timeout_add_seconds(1, call_timer, win);
+	pthread_mutex_lock(&last_data_mut);
 	last_call_win = win;
+	pthread_mutex_unlock(&last_data_mut);
 	call_window_set_status(win, "progress");
 }
 
@@ -533,7 +563,9 @@ void call_window_established(struct call_window *win)
 								win);
 	}
 
+	pthread_mutex_lock(&last_data_mut);
 	last_call_win = win;
+	pthread_mutex_unlock(&last_data_mut);
 	call_window_set_status(win, "established");
 }
 
