@@ -2,7 +2,7 @@
  * @file gtk/gtk_mod.c GTK+ UI module
  *
  * Copyright (C) 2015 Charles E. Lehner
- * Copyright (C) 2010 - 2015 Creytiv.com
+ * Copyright (C) 2010 - 2015 Alfred E. Heggestad
  */
 #include <re.h>
 #include <rem.h>
@@ -22,9 +22,9 @@
 #endif
 
 /* About */
-#define COPYRIGHT " Copyright (C) 2010 - 2019 Alfred E. Heggestad et al."
+#define COPYRIGHT " Copyright (C) 2010 - 2021 Alfred E. Heggestad et al."
 #define COMMENTS "A modular SIP User-Agent with audio and video support"
-#define WEBSITE "http://www.creytiv.com/baresip.html"
+#define WEBSITE "https://github.com/baresip/baresip"
 #define LICENSE "BSD"
 
 
@@ -54,6 +54,7 @@ struct gtk_mod {
 	GSList *call_windows;
 	GSList *incoming_call_menus;
 	bool clean_number;
+	struct ua *ua_cur;
 };
 
 static struct gtk_mod mod_obj;
@@ -77,9 +78,24 @@ static GActionEntry app_entries[] = {
 };
 
 
+static void gtk_current_ua_set(struct ua *ua)
+{
+	mod_obj.ua_cur = ua;
+}
+
+
+static struct ua *gtk_current_ua(void)
+{
+	if (!mod_obj.ua_cur)
+		mod_obj.ua_cur = list_ledata(list_head(uag_list()));
+
+	return mod_obj.ua_cur;
+}
+
+
 static struct call *get_call_from_gvariant(GVariant *param)
 {
-	struct list *calls = ua_calls(uag_current());
+	struct list *calls = ua_calls(gtk_current_ua());
 	const gchar *call_ptr = g_variant_get_string(param, NULL);
 
 	return call_find_id(calls, call_ptr);
@@ -131,7 +147,7 @@ static void menu_on_dial_contact(GtkMenuItem *menuItem, gpointer arg)
 	struct gtk_mod *mod = arg;
 	const char *uri = gtk_menu_item_get_label(menuItem);
 	/* Queue dial from the main thread */
-	mqueue_push(mod->mq, MQ_CONNECT, (char *)uri);
+	gtk_mod_connect(mod, uri);
 }
 
 
@@ -158,7 +174,7 @@ static void menu_on_account_toggled(GtkCheckMenuItem *menu_item,
 				    struct gtk_mod *mod)
 {
 	struct ua *ua = g_object_get_data(G_OBJECT(menu_item), "ua");
-	if (menu_item->active)
+	if (gtk_check_menu_item_get_active(menu_item))
 		mqueue_push(mod->mq, MQ_SELECT_UA, ua);
 }
 
@@ -203,10 +219,10 @@ static GtkMenuItem *accounts_menu_add_item(struct gtk_mod *mod,
 	GtkMenuShell *accounts_menu = GTK_MENU_SHELL(mod->accounts_menu);
 	GtkWidget *item;
 	GSList *group = mod->accounts_menu_group;
-	struct ua *ua_current = uag_current();
+	struct ua *ua_current = gtk_current_ua();
 	char buf[256];
 
-	re_snprintf(buf, sizeof buf, "%s%s", ua_aor(ua),
+	re_snprintf(buf, sizeof buf, "%s%s", account_aor(ua_account(ua)),
 			ua_isregistered(ua) ? " (OK)" : "");
 	item = gtk_radio_menu_item_new_with_label(group, buf);
 	group = gtk_radio_menu_item_get_group(
@@ -228,8 +244,8 @@ static GtkMenuItem *accounts_menu_get_item(struct gtk_mod *mod,
 					   struct ua *ua)
 {
 	GtkMenuItem *item;
-	GtkMenuShell *accounts_menu = GTK_MENU_SHELL(mod->accounts_menu);
-	GList *items = accounts_menu->children;
+	GtkContainer *accounts_menu_cont = GTK_CONTAINER(mod->accounts_menu);
+	GList *items = gtk_container_get_children(accounts_menu_cont);
 
 	for (; items; items = items->next) {
 		item = items->data;
@@ -244,7 +260,7 @@ static GtkMenuItem *accounts_menu_get_item(struct gtk_mod *mod,
 
 static void update_current_accounts_menu_item(struct gtk_mod *mod)
 {
-	GtkMenuItem *item = accounts_menu_get_item(mod, uag_current());
+	GtkMenuItem *item = accounts_menu_get_item(mod, gtk_current_ua());
 
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), TRUE);
 }
@@ -255,10 +271,10 @@ static void update_ua_presence(struct gtk_mod *mod)
 	GtkCheckMenuItem *item = 0;
 	enum presence_status cur_status;
 	void *status;
-	GtkMenuShell *status_menu = GTK_MENU_SHELL(mod->status_menu);
-	GList *items = status_menu->children;
+	GtkContainer *status_menu_cont = GTK_CONTAINER(mod->status_menu);
+	GList *items = gtk_container_get_children(status_menu_cont);
 
-	cur_status = ua_presence_status(uag_current());
+	cur_status = ua_presence_status(gtk_current_ua());
 
 	for (; items; items = items->next) {
 		item = items->data;
@@ -291,7 +307,7 @@ static void accounts_menu_set_status(struct gtk_mod *mod,
 {
 	GtkMenuItem *item = accounts_menu_get_item(mod, ua);
 	char buf[256];
-	re_snprintf(buf, sizeof buf, "%s (%s)", ua_aor(ua),
+	re_snprintf(buf, sizeof buf, "%s (%s)", account_aor(ua_account(ua)),
 			ua_event_reg_str(ev));
 	gtk_menu_item_set_label(item, buf);
 }
@@ -513,8 +529,7 @@ static void ua_event_handler(struct ua *ua,
 		win = get_call_window(mod, call);
 		if (win)
 			call_window_closed(win, prm);
-		else
-			denotify_incoming_call(mod, call);
+		denotify_incoming_call(mod, call);
 		break;
 
 	case UA_EVENT_CALL_RINGING:
@@ -533,6 +548,7 @@ static void ua_event_handler(struct ua *ua,
 		win = get_create_call_window(mod, call);
 		if (win)
 			call_window_established(win);
+		denotify_incoming_call(mod, call);
 		break;
 
 	case UA_EVENT_CALL_TRANSFER_FAILED:
@@ -625,14 +641,36 @@ static gboolean status_icon_on_button_press(GtkStatusIcon *status_icon,
 }
 
 
-void gtk_mod_connect(struct gtk_mod *mod, const char *uri)
+int gtk_mod_connect(struct gtk_mod *mod, const char *uri)
 {
-	if (!mod)
-		return;
+	struct mbuf *uribuf = NULL;
+	char *uri_copy = NULL;
+	int err = 0;
 
-	mqueue_push(mod->mq, MQ_CONNECT, (char *)uri);
+	if (!mod)
+		return ENOMEM;
+
+	uribuf = mbuf_alloc(64);
+	if (!uribuf)
+		return ENOMEM;
+
+	err = account_uri_complete(ua_account(mod->ua_cur), uribuf, uri);
+	if (err)
+		return EINVAL;
+
+	uribuf->pos = 0;
+	err = mbuf_strdup(uribuf, &uri_copy, uribuf->end);
+	if (err)
+		goto out;
+
+	err = mqueue_push(mod->mq, MQ_CONNECT, uri_copy);
+
+out:
+	mem_deref(uribuf);
+	return err;
 }
 
+
 bool gtk_mod_clean_number(struct gtk_mod *mod)
 {
 	if (!mod)
@@ -668,7 +706,7 @@ static void mqueue_handler(int id, void *data, void *arg)
 	const char *uri;
 	struct call *call;
 	int err;
-	struct ua *ua = uag_current();
+	struct ua *ua = gtk_current_ua();
 
 	switch ((enum gtk_mod_events)id) {
 
@@ -695,6 +733,7 @@ static void mqueue_handler(int id, void *data, void *arg)
 		if (err) {
 			ua_hangup(ua, call, 500, "Server Error");
 		}
+		mem_deref(data);
 		break;
 
 	case MQ_HANGUP:
@@ -730,7 +769,7 @@ static void mqueue_handler(int id, void *data, void *arg)
 
 	case MQ_SELECT_UA:
 		ua = data;
-		uag_current_set(ua);
+		gtk_current_ua_set(ua);
 		break;
 	}
 }
@@ -748,7 +787,7 @@ static void *gtk_thread(void *arg)
 	gtk_init(0, NULL);
 
 	g_set_application_name("baresip");
-	mod->app = g_application_new("com.creytiv.baresip",
+	mod->app = g_application_new("com.github.baresip",
 				     G_APPLICATION_FLAGS_NONE);
 
 	g_application_register(G_APPLICATION (mod->app), NULL, &err);
