@@ -1,7 +1,7 @@
 /**
  * @file opensles/player.c  OpenSLES audio driver -- playback
  *
- * Copyright (C) 2010 Creytiv.com
+ * Copyright (C) 2010 Alfred E. Heggestad
  */
 #include <re.h>
 #include <rem.h>
@@ -16,7 +16,6 @@
 
 
 struct auplay_st {
-	const struct auplay *ap;      /* inheritance */
 	auplay_write_h *wh;
 	void *arg;
 	int16_t *sampv[N_PLAY_QUEUE_BUFFERS];
@@ -50,11 +49,15 @@ static void auplay_destructor(void *arg)
 static void bqPlayerCallback(SLAndroidSimpleBufferQueueItf bq, void *context)
 {
 	struct auplay_st *st = context;
+	struct auframe af;
 
-	st->wh(st->sampv[st->bufferId], st->sampc, st->arg);
+	auframe_init(&af, AUFMT_S16LE, st->sampv[st->bufferId], st->sampc);
+
+	st->wh(&af, st->arg);
 
 	(*st->BufferQueue)->Enqueue(bq /*st->BufferQueue*/,
-				    st->sampv[st->bufferId], st->sampc * 2);
+				    st->sampv[st->bufferId],
+				    (unsigned int)(st->sampc * 2));
 
 	st->bufferId = ( st->bufferId + 1 ) % N_PLAY_QUEUE_BUFFERS;
 }
@@ -167,7 +170,6 @@ int opensles_player_alloc(struct auplay_st **stp, const struct auplay *ap,
 	if (!st)
 		return ENOMEM;
 
-	st->ap  = ap;
 	st->wh  = wh;
 	st->arg = arg;
 
