@@ -1,7 +1,7 @@
 /**
  * @file encode.cpp  WebRTC Acoustic Echo Cancellation (AEC) -- Encode
  *
- * Copyright (C) 2010 Creytiv.com
+ * Copyright (C) 2010 Alfred E. Heggestad
  */
 
 #include <string.h>
@@ -22,7 +22,6 @@ struct aec_enc {
 
 	struct aec *aec;
 	float buf[160];
-	enum aufmt fmt;
 };
 
 
@@ -64,8 +63,6 @@ int webrtc_aec_encode_update(struct aufilt_enc_st **stp, void **ctx,
 	if (!st)
 		return ENOMEM;
 
-	st->fmt = (enum aufmt)prm->fmt;
-
 	err = webrtc_aec_alloc(&st->aec, ctx, prm);
 	if (err)
 		goto out;
@@ -128,7 +125,7 @@ int webrtc_aec_encode(struct aufilt_enc_st *st, struct auframe *af)
 	if (!st || !af)
 		return EINVAL;
 
-	switch (enc->fmt) {
+	switch (af->fmt) {
 
 	case AUFMT_S16LE:
 		/* convert from S16 to FLOAT */
