@@ -1,12 +1,13 @@
 /**
  * @file src/account.c  User-Agent account
  *
- * Copyright (C) 2010 Creytiv.com
+ * Copyright (C) 2010 Alfred E. Heggestad
  */
 #include <string.h>
 #include <re.h>
 #include <baresip.h>
 #include "core.h"
+#include <ctype.h>
 
 
 enum {
@@ -99,13 +100,9 @@ static int stunsrv_decode(struct account *acc, const struct sip_addr *aor)
 
 	if (0 == msg_param_exists(&aor->params, "stunuser", &tmp))
 		err |= param_dstr(&acc->stun_user, &aor->params, "stunuser");
-	else
-		err |= pl_strdup(&acc->stun_user, &aor->uri.user);
 
 	if (0 == msg_param_exists(&aor->params, "stunpass", &tmp))
 		err |= param_dstr(&acc->stun_pass, &aor->params, "stunpass");
-	else if (acc->auth_pass)
-		err |= str_dup(&acc->stun_pass, acc->auth_pass);
 
 	return err;
 }
@@ -167,7 +164,7 @@ static int decode_pair(char **val1, char **val2,
 /* Decode answermode parameter */
 static void answermode_decode(struct account *prm, const struct pl *pl)
 {
-	struct pl amode;
+	struct pl amode, adelay;
 
 	prm->answermode = ANSWERMODE_MANUAL;
 
@@ -186,6 +183,38 @@ static void answermode_decode(struct account *prm, const struct pl *pl)
 			warning("account: answermode unknown (%r)\n", &amode);
 		}
 	}
+
+	if (0 == msg_param_decode(pl, "answerdelay", &adelay))
+		prm->adelay = pl_u32(&adelay);
+}
+
+
+static void autoanswer_allow_decode(struct account *prm, const struct pl *pl)
+{
+	struct pl v;
+
+	if (0 == msg_param_decode(pl, "sip_autoanswer", &v)) {
+		if (0 == pl_strcasecmp(&v, "yes")) {
+			prm->sipans = true;
+		}
+	}
+}
+
+
+/* Decode dtmfmode parameter */
+static void dtmfmode_decode(struct account *prm, const struct pl *pl)
+{
+	struct pl dtmfmode;
+
+	if (0 == msg_param_decode(pl, "dtmfmode", &dtmfmode)) {
+
+		if (0 == pl_strcasecmp(&dtmfmode, "info")) {
+			prm->dtmfmode = DTMFMODE_SIP_INFO;
+		}
+		else {
+			prm->dtmfmode = DTMFMODE_RTP_EVENT;
+		}
+	}
 }
 
 
@@ -283,21 +312,21 @@ static int video_codecs_decode(struct account *acc, const struct pl *prm)
 			return 0;
 
 		while (0 == csl_parse(&vcs, cname, sizeof(cname))) {
-			struct vidcodec *vc;
+			struct le *le;
 
-			vc = (struct vidcodec *)vidcodec_find(vidcodecl,
-							      cname, NULL);
-			if (!vc) {
-				warning("account: video codec not found: %s\n",
-					cname);
-				continue;
+			for (le=list_head(vidcodecl); le; le=le->next) {
+				struct vidcodec *vc = le->data;
+
+				if (0 != str_casecmp(cname, vc->name))
+					continue;
+
+				/* static list with references to vidcodec */
+				list_append(&acc->vidcodecl, &acc->vcv[i++],
+						vc);
+
+				if (i >= ARRAY_SIZE(acc->vcv))
+					return 0;
 			}
-
-			/* NOTE: static list with references to vidcodec */
-			list_append(&acc->vidcodecl, &acc->vcv[i++], vc);
-
-			if (i >= ARRAY_SIZE(acc->vcv))
-				break;
 		}
 	}
 
@@ -316,6 +345,7 @@ static int sip_params_decode(struct account *acc, const struct sip_addr *aor)
 
 	acc->regint = REG_INTERVAL + (rand_u32()&0xff);
 	err |= param_u32(&acc->regint, &aor->params, "regint");
+	err |= param_u32(&acc->prio, &aor->params, "prio");
 	err |= param_u32(&acc->rwait, &aor->params, "rwait");
 	if (acc->rwait > 95)
 		acc->rwait = 95;
@@ -323,6 +353,7 @@ static int sip_params_decode(struct account *acc, const struct sip_addr *aor)
 	if (acc->rwait && acc->rwait < 5)
 		acc->rwait = 5;
 
+	err |= param_u32(&acc->fbregint, &aor->params, "fbregint");
 	acc->pubint = 0;
 	err |= param_u32(&acc->pubint, &aor->params, "pubint");
 
@@ -419,6 +450,8 @@ int account_alloc(struct account **accp, const char *sipaddr)
 	acc->ptime = 20;
 	err |= sip_params_decode(acc, &acc->laddr);
 	       answermode_decode(acc, &acc->laddr.params);
+	       autoanswer_allow_decode(acc, &acc->laddr.params);
+	       dtmfmode_decode(acc,&acc->laddr.params);
 	err |= audio_codecs_decode(acc, &acc->laddr.params);
 	err |= video_codecs_decode(acc, &acc->laddr.params);
 	err |= media_decode(acc, &acc->laddr.params);
@@ -987,7 +1020,7 @@ struct uri *account_luri(const struct account *acc)
 
 
 /**
- * Get the Registration interval of an account
+ * Get the registration interval of an account
  *
  * @param acc User-Agent account
  *
@@ -999,6 +1032,32 @@ uint32_t account_regint(const struct account *acc)
 }
 
 
+/**
+ * Get the fallback registration interval of an account
+ *
+ * @param acc User-Agent account
+ *
+ * @return Registration interval in [seconds]
+ */
+uint32_t account_fbregint(const struct account *acc)
+{
+	return acc ? acc->fbregint : 0;
+}
+
+
+/**
+ * Get the priority of an account. Priority 0 is default.
+ *
+ * @param acc User-Agent account
+ *
+ * @return Priority
+ */
+uint32_t account_prio(const struct account *acc)
+{
+	return acc ? acc->prio : 0;
+}
+
+
 /**
  * Get the Publication interval of an account
  *
@@ -1050,6 +1109,43 @@ int account_set_answermode(struct account *acc, enum answermode mode)
 }
 
 
+/**
+ * Get the dtmfmode of an account
+ *
+ * @param acc User-Agent account
+ *
+ * @return Dtmfmode
+ */
+enum dtmfmode account_dtmfmode(const struct account *acc)
+{
+	return acc ? acc->dtmfmode : DTMFMODE_RTP_EVENT;
+}
+
+
+/**
+ * Set the dtmfmode of an account
+ *
+ * @param acc  User-Agent account
+ * @param mode Dtmfmode
+ *
+ * @return 0 if success, otherwise errorcode
+ */
+int account_set_dtmfmode(struct account *acc, enum dtmfmode mode)
+{
+	if (!acc)
+		return EINVAL;
+
+	if ((mode != DTMFMODE_RTP_EVENT) && (mode != DTMFMODE_SIP_INFO)) {
+		warning("account: invalid dtmfmode : `%d'\n", mode);
+		return EINVAL;
+	}
+
+	acc->dtmfmode = mode;
+
+	return 0;
+}
+
+
 /**
  * Get the SIP Display Name of an account
  *
@@ -1219,6 +1315,18 @@ uint16_t account_stun_port(const struct account *acc)
 }
 
 
+/**
+ * Returns if SIP auto answer is allowed for the account
+ *
+ * @param acc User-Agent account
+ * @return true if allowed, otherwise false
+ */
+bool account_sip_autoanswer(const struct account *acc)
+{
+	return acc ? acc->sipans : false;
+}
+
+
 static const char *answermode_str(enum answermode mode)
 {
 	switch (mode) {
@@ -1231,6 +1339,17 @@ static const char *answermode_str(enum answermode mode)
 }
 
 
+static const char *dtmfmode_str(enum dtmfmode mode)
+{
+	switch (mode) {
+
+	case DTMFMODE_RTP_EVENT: return "rtpevent";
+	case DTMFMODE_SIP_INFO:  return "info";
+	default: return "???";
+	}
+}
+
+
 /**
  * Get the media encryption of an account
  *
@@ -1302,6 +1421,93 @@ const char *account_extra(const struct account *acc)
 }
 
 
+/**
+ * Auto complete a SIP uri, add scheme and domain if missing
+ *
+ * @param acc User-Agent account
+ * @param buf Target buffer to print SIP uri
+ * @param uri Input SIP uri
+ *
+ * @return 0 if success, otherwise errorcode
+ */
+int account_uri_complete(const struct account *acc, struct mbuf *buf,
+			 const char *uri)
+{
+	struct sa sa_addr;
+	size_t len;
+	bool uri_is_ip;
+	char *uridup;
+	char *host;
+	char *param;
+	int err = 0;
+
+	if (!buf || !uri)
+		return EINVAL;
+
+	/* Skip initial whitespace */
+	while (isspace(*uri))
+		++uri;
+
+	len = str_len(uri);
+
+	/* Append sip: scheme if missing */
+	if (0 != re_regex(uri, len, "sip:"))
+		err |= mbuf_printf(buf, "sip:");
+
+	err |= mbuf_write_str(buf, uri);
+
+	if (!acc || err)
+		return err;
+
+	/* Append domain if missing and uri is not IP address */
+
+	/* check if uri is valid IP address */
+	err = str_dup(&uridup, uri);
+	if (err)
+		return err;
+
+	if (!strncmp(uridup, "sip:", 4))
+		host = uridup + 4;
+	else
+		host = uridup;
+
+	param = strchr(host, ';');
+	if (param)
+		*param = 0;
+
+	uri_is_ip =
+		!sa_decode(&sa_addr, host, strlen(host)) ||
+		!sa_set_str(&sa_addr, host, 0);
+	mem_deref(uridup);
+
+	if (0 != re_regex(uri, len, "[^@]+@[^]+", NULL, NULL) &&
+		1 != uri_is_ip) {
+#if HAVE_INET6
+		if (AF_INET6 == acc->luri.af)
+			err |= mbuf_printf(buf, "@[%r]",
+					   &acc->luri.host);
+		else
+#endif
+			err |= mbuf_printf(buf, "@%r",
+					   &acc->luri.host);
+
+		/* Also append port if specified and not 5060 */
+		switch (acc->luri.port) {
+
+		case 0:
+		case SIP_PORT:
+			break;
+
+		default:
+			err |= mbuf_printf(buf, ":%u", acc->luri.port);
+			break;
+		}
+	}
+
+	return err;
+}
+
+
 /**
  * Print the account debug information
  *
@@ -1328,6 +1534,10 @@ int account_debug(struct re_printf *pf, const struct account *acc)
 	err |= re_hprintf(pf, " dispname:     %s\n", acc->dispname);
 	err |= re_hprintf(pf, " answermode:   %s\n",
 			  answermode_str(acc->answermode));
+	err |= re_hprintf(pf, " sipans:       %s\n",
+			  acc->sipans ? "yes" : "no");
+	err |= re_hprintf(pf, " dtmfmode:     %s\n",
+			  dtmfmode_str(acc->dtmfmode));
 	if (!list_isempty(&acc->aucodecl)) {
 		err |= re_hprintf(pf, " audio_codecs:");
 		for (le = list_head(&acc->aucodecl); le; le = le->next) {
@@ -1351,6 +1561,7 @@ int account_debug(struct re_printf *pf, const struct account *acc)
 	err |= re_hprintf(pf, " mwi:          %s\n", account_mwi(acc));
 	err |= re_hprintf(pf, " ptime:        %u\n", acc->ptime);
 	err |= re_hprintf(pf, " regint:       %u\n", acc->regint);
+	err |= re_hprintf(pf, " prio:         %u\n", acc->prio);
 	err |= re_hprintf(pf, " pubint:       %u\n", acc->pubint);
 	err |= re_hprintf(pf, " regq:         %s\n", acc->regq);
 	err |= re_hprintf(pf, " sipnat:       %s\n", acc->sipnat);
@@ -1390,6 +1601,7 @@ int account_json_api(struct odict *od, struct odict *odcfg,
 	int err = 0;
 	struct odict *obn = NULL;
 	const char *stunhost = "";
+	size_t i;
 
 	if (!acc)
 		return 0;
@@ -1408,7 +1620,7 @@ int account_json_api(struct odict *od, struct odict *odcfg,
 	}
 
 	err |= odict_alloc(&obn, 8);
-	for (size_t i=0; i<ARRAY_SIZE(acc->outboundv); i++) {
+	for (i=0; i<ARRAY_SIZE(acc->outboundv); i++) {
 		if (acc->outboundv[i]) {
 			err |= odict_entry_add(obn, "outbound", ODICT_STRING,
 					acc->outboundv[i]);
