@@ -1,7 +1,7 @@
 /**
  * @file src/call.c  Call Control
  *
- * Copyright (C) 2010 Creytiv.com
+ * Copyright (C) 2010 Alfred E. Heggestad
  */
 #include <stdio.h>
 #include <stdlib.h>
@@ -37,6 +37,7 @@ struct call {
 	struct video *video;      /**< Video stream                         */
 	struct media_ctx *ctx;    /**< Shared A/V source media context      */
 	enum call_state state;    /**< Call state                           */
+	int32_t adelay;           /**< Auto answer delay in ms              */
 	char *local_uri;          /**< Local SIP uri                        */
 	char *local_name;         /**< Local display name                   */
 	char *peer_uri;           /**< Peer SIP Address                     */
@@ -44,6 +45,7 @@ struct call {
 	char *id;                 /**< Cached session call-id               */
 	struct tmr tmr_inv;       /**< Timer for incoming calls             */
 	struct tmr tmr_dtmf;      /**< Timer for incoming DTMF events       */
+	struct tmr tmr_answ;      /**< Timer for delayed answer             */
 	time_t time_start;        /**< Time when call started               */
 	time_t time_conn;         /**< Time when call initiated             */
 	time_t time_stop;         /**< Time when call stopped               */
@@ -69,6 +71,7 @@ struct call {
 
 
 static int send_invite(struct call *call);
+static int send_dtmf_info(struct call *call, char key);
 
 
 static const char *state_name(enum call_state st)
@@ -161,39 +164,6 @@ static int start_audio(struct call *call)
 }
 
 
-static int start_video(struct call *call)
-{
-	const struct sdp_format *sc;
-	int err = 0;
-
-	/* Video Stream */
-	sc = sdp_media_rformat(stream_sdpmedia(video_strm(call->video)), NULL);
-	if (sc) {
-		err  = video_encoder_set(call->video, sc->data, sc->pt,
-					 sc->params);
-		err |= video_decoder_set(call->video, sc->data, sc->pt,
-					 sc->rparams);
-		if (err)
-			return err;
-
-		if (!video_is_started(call->video)) {
-			err  = video_start_display(call->video,
-						   call->peer_uri);
-			err |= video_start_source(call->video, &call->ctx);
-		}
-
-		if (err) {
-			warning("call: video stream error: %m\n", err);
-		}
-	}
-	else if (call->video) {
-		info("call: video stream is disabled..\n");
-	}
-
-	return err;
-}
-
-
 static void call_stream_start(struct call *call, bool active)
 {
 	int err;
@@ -208,7 +178,7 @@ static void call_stream_start(struct call *call, bool active)
 	}
 
 	if (stream_is_ready(video_strm(call->video))) {
-		err = start_video(call);
+		err = video_update(call->video, &call->ctx, call->peer_uri);
 		if (err) {
 			warning("call: could not start video: %m\n", err);
 		}
@@ -238,7 +208,7 @@ static void call_stream_stop(struct call *call)
 	audio_stop(call->audio);
 
 	/* Video */
-	video_stop(call->video);
+	video_stop(call->video, &call->ctx);
 
 	tmr_cancel(&call->tmr_inv);
 }
@@ -327,6 +297,8 @@ static int update_audio(struct call *call)
 	const struct sdp_format *sc;
 	int err = 0;
 
+	debug("audio: update\n");
+
 	sc = sdp_media_rcodec(stream_sdpmedia(audio_strm(call->audio)));
 	if (sc) {
 		struct aucodec *ac = sc->data;
@@ -348,44 +320,6 @@ static int update_audio(struct call *call)
 }
 
 
-static int update_video(struct call *call)
-{
-	const struct sdp_format *sc = NULL;
-	int err = 0;
-
-	struct sdp_media *m = stream_sdpmedia(video_strm(call->video));
-
-	if (!sdp_media_disabled(m))
-		sc = sdp_media_rformat(m, NULL);
-
-	if (sc) {
-		err = video_encoder_set(call->video, sc->data,
-					sc->pt, sc->params);
-		if (err) {
-			warning("call: video stream error: %m\n", err);
-			return err;
-		}
-
-		if (!video_is_started(call->video)) {
-			err  = video_start_display(call->video,
-						   call->peer_uri);
-			err |= video_start_source(call->video, &call->ctx);
-			if (err) {
-				warning("call: update: failed to"
-					" start video (%m)\n", err);
-			}
-		}
-	}
-	else if (call->video) {
-		info("video stream is disabled..\n");
-		video_stop(call->video);
-		video_stop_display(call->video);
-	}
-
-	return err;
-}
-
-
 static int update_media(struct call *call)
 {
 	struct le *le;
@@ -415,7 +349,7 @@ static int update_media(struct call *call)
 	}
 
 	if (stream_is_ready(video_strm(call->video))) {
-		err |= update_video(call);
+		err |= video_update(call->video, &call->ctx, call->peer_uri);
 	}
 
 	return err;
@@ -443,6 +377,7 @@ static void call_destructor(void *arg)
 	call_stream_stop(call);
 	list_unlink(&call->le);
 	tmr_cancel(&call->tmr_dtmf);
+	tmr_cancel(&call->tmr_answ);
 
 	mem_deref(call->sess);
 	mem_deref(call->id);
@@ -468,7 +403,7 @@ static void audio_event_handler(int key, bool end, void *arg)
 	struct call *call = arg;
 	MAGIC_CHECK(call);
 
-	info("received event: '%c' (end=%d)\n", key, end);
+	info("received in-band DTMF event: '%c' (end=%d)\n", key, end);
 
 	if (call->dtmfh)
 		call->dtmfh(call, end ? KEYCODE_REL : key, call->arg);
@@ -537,7 +472,8 @@ static void menc_event_handler(enum menc_event event,
 		else if (strstr(prm, "video")) {
 			stream_set_secure(video_strm(call->video), true);
 			stream_start(video_strm(call->video));
-			err = start_video(call);
+			err = video_update(call->video, &call->ctx,
+				call->peer_uri);
 			if (err) {
 				warning("call: secure: could not"
 					" start video: %m\n", err);
@@ -599,7 +535,8 @@ static void stream_mnatconn_handler(struct stream *strm, void *arg)
 			break;
 
 		case MEDIA_VIDEO:
-			err = start_video(call);
+			err = video_update(call->video, &call->ctx,
+				call->peer_uri);
 			if (err) {
 				warning("call: mnatconn: could not"
 					" start video: %m\n", err);
@@ -677,6 +614,101 @@ static int assign_linenum(uint32_t *linenum, const struct list *lst)
 }
 
 
+/**
+ * Decode the SIP-Header for RFC 5373 auto answer of incoming call
+ *
+ * @param call Call object
+ * @param msg  SIP message
+ * @param name SIP header name
+ */
+static void call_rfc5373_autoanswer(struct call *call,
+		const struct sip_msg *msg, const char *name)
+{
+	const struct sip_hdr *hdr;
+	struct pl v1;
+
+	hdr = sip_msg_xhdr(msg, name);
+	if (!hdr || pl_strcasecmp(&hdr->val, "Auto"))
+		return;
+
+	if (!msg_param_exists(&hdr->val, "require", &v1) &&
+			!account_sip_autoanswer(call->acc)) {
+
+		warning("call: rejected, since %s is not allowed\n", name);
+		call_hangup(call, 0, NULL);
+		return;
+	}
+
+	call->adelay = 0;
+}
+
+
+/**
+ * Decodes given SIP header for auto answer options of incoming call
+ *
+ * @param call Call object
+ * @param hdr  SIP header (Call-Info or Alert-Info)
+ * @return true if success, otherwise false
+ */
+static bool call_hdr_dec_sip_autoanswer(struct call *call,
+		const struct sip_hdr *hdr)
+{
+	struct pl v1, v2;
+	if (!call || !hdr)
+		return false;
+
+	if (!msg_param_decode(&hdr->val, "answer-after", &v1)) {
+		call->adelay = pl_u32(&v1) * 1000;
+		return true;
+	}
+
+	if (!msg_param_decode(&hdr->val, "info", &v1) &&
+			!msg_param_decode(&hdr->val, "delay", &v2)) {
+		if (!pl_strcmp(&v1, "alert-autoanswer")) {
+			call->adelay = pl_u32(&v2) * 1000;
+			return true;
+		}
+	}
+
+	if (!msg_param_decode(&hdr->val, "info", &v1)) {
+		if (!pl_strcmp(&v1, "alert-autoanswer")) {
+			call->adelay = 0;
+			return true;
+		}
+	}
+
+	return false;
+}
+
+
+/**
+ * Decode the SIP message for auto answer options of incoming call
+ *
+ * @param call Call object
+ * @param msg  SIP message
+ */
+static void call_decode_sip_autoanswer(struct call *call,
+		const struct sip_msg *msg)
+{
+	const struct sip_hdr *hdr;
+
+	call->adelay = -1;
+
+	/* polycom (HDA50), avaya, grandstream, snom, gigaset, yealink */
+	hdr = sip_msg_hdr(msg, SIP_HDR_CALL_INFO);
+	if (call_hdr_dec_sip_autoanswer(call, hdr))
+		return;
+
+	hdr = sip_msg_hdr(msg, SIP_HDR_ALERT_INFO);
+	if (call_hdr_dec_sip_autoanswer(call, hdr))
+		return;
+
+	/* RFC 5373 */
+	call_rfc5373_autoanswer(call, msg, "Answer-Mode");
+	call_rfc5373_autoanswer(call, msg, "Priv-Answer-Mode");
+}
+
+
 /**
  * Allocate a new Call state object
  *
@@ -732,6 +764,7 @@ int call_alloc(struct call **callp, const struct config *cfg, struct list *lst,
 	call->config_call = cfg->call;
 
 	tmr_init(&call->tmr_inv);
+	tmr_init(&call->tmr_answ);
 
 	call->acc    = mem_ref(acc);
 	call->ua     = ua;
@@ -739,6 +772,7 @@ int call_alloc(struct call **callp, const struct config *cfg, struct list *lst,
 	call->eh     = eh;
 	call->arg    = arg;
 	call->af     = prm->af;
+	call_decode_sip_autoanswer(call, msg);
 
 	err = str_dup(&call->local_uri, local_uri);
 	if (local_name)
@@ -751,11 +785,6 @@ int call_alloc(struct call **callp, const struct config *cfg, struct list *lst,
 	if (err)
 		goto out;
 
-	err = sdp_session_set_lattr(call->sdp, true,
-				    "tool", "baresip " BARESIP_VERSION);
-	if (err)
-		goto out;
-
 	/* Check for incoming SDP Offer */
 	if (msg && mbuf_get_left(msg->mb))
 		got_offer = true;
@@ -823,7 +852,7 @@ int call_alloc(struct call **callp, const struct config *cfg, struct list *lst,
 				  video_error_handler, call);
 		if (err)
 			goto out;
- 	}
+	}
 
 	/* inherit certain properties from original call */
 	if (xcall) {
@@ -974,9 +1003,15 @@ int call_modify(struct call *call)
 	debug("call: modify\n");
 
 	err = call_sdp_get(call, &desc, true);
-	if (!err)
+	if (!err) {
 		err = sipsess_modify(call->sess, desc);
+		if (err)
+			goto out;
+	}
 
+	err = update_media(call);
+
+ out:
 	mem_deref(desc);
 
 	return err;
@@ -1079,6 +1114,8 @@ int call_answer(struct call *call, uint16_t scode, enum vidmode vmode)
 	if (!call || !call->sess)
 		return EINVAL;
 
+	tmr_cancel(&call->tmr_answ);
+
 	if (CALL_STATE_INCOMING != call->state) {
 		info("call: answer: call is not in incoming state (%s)\n",
 		     state_name(call->state));
@@ -1276,10 +1313,12 @@ int call_debug(struct re_printf *pf, const struct call *call)
 	err |= re_hprintf(pf,
 			  " local_uri: %s <%s>\n"
 			  " peer_uri:  %s <%s>\n"
-			  " af=%s id=%s\n",
+			  " af=%s id=%s\n"
+			  " autoanswer delay: %d\n",
 			  call->local_name, call->local_uri,
 			  call->peer_name, call->peer_uri,
-			  net_af2name(call->af), call->id);
+			  net_af2name(call->af), call->id,
+			  call->adelay);
 	err |= re_hprintf(pf, " direction: %s\n",
 			  call->outgoing ? "Outgoing" : "Incoming");
 
@@ -1363,7 +1402,8 @@ int call_info(struct re_printf *pf, const struct call *call)
 	if (!call)
 		return 0;
 
-	return re_hprintf(pf, "[line %u]  %H  %9s  %s  %s", call->linenum,
+	return re_hprintf(pf, "[line %u, id %s]  %H  %9s  %s  %s",
+			  call->linenum, call->id,
 			  print_duration, call,
 			  state_name(call->state),
 			  call->on_hold ? "(on hold)" : "         ",
@@ -1381,10 +1421,20 @@ int call_info(struct re_printf *pf, const struct call *call)
  */
 int call_send_digit(struct call *call, char key)
 {
+	int err;
+
 	if (!call)
 		return EINVAL;
 
-	return audio_send_digit(call->audio, key);
+	if (call->acc->dtmfmode == DTMFMODE_SIP_INFO &&
+		key != KEYCODE_REL) {
+		err = send_dtmf_info(call, key);
+	}
+	else {
+		err = audio_send_digit(call->audio, key);
+	}
+
+	return err;
 }
 
 
@@ -1420,12 +1470,12 @@ static int sipsess_offer_handler(struct mbuf **descp,
 {
 	const bool got_offer = (0 != mbuf_get_left(msg->mb));
 	struct call *call = arg;
+	struct sdp_media *vmedia = NULL;
+	enum sdp_dir ardir, vrdir;
 	int err;
 
 	MAGIC_CHECK(call);
 
-	info("call: got re-INVITE%s\n", got_offer ? " (SDP Offer)" : "");
-
 	if (got_offer) {
 
 		call->got_offer = true;
@@ -1439,10 +1489,29 @@ static int sipsess_offer_handler(struct mbuf **descp,
 		}
 
 		err = update_media(call);
-		if (err)
+		if (err) {
+			warning("call: reinvite: could not update media: %m\n",
+				err);
 			return err;
+		}
 	}
 
+	ardir = sdp_media_rdir(
+		stream_sdpmedia(audio_strm(call_audio(call))));
+
+	vmedia = stream_sdpmedia(video_strm(call_video(call)));
+	if (sdp_media_rport(vmedia) == 0 ||
+		list_head(sdp_media_format_lst(vmedia, false)) == 0) {
+		vrdir = SDP_INACTIVE;
+	}
+	else {
+		vrdir = sdp_media_rdir(vmedia);
+	}
+
+	info("call: got re-INVITE%s audio-video: %s-%s\n"
+		, got_offer ? " (SDP Offer)" : "",
+		sdp_dir_name(ardir), sdp_dir_name(vrdir));
+
 	/* Encode SDP Answer */
 	return sdp_encode(descp, call->sdp, !got_offer);
 }
@@ -1458,6 +1527,7 @@ static int sipsess_answer_handler(const struct sip_msg *msg, void *arg)
 	debug("call: got SDP answer (%zu bytes)\n", mbuf_get_left(msg->mb));
 
 	call->got_offer = false;
+	call_event_handler(call, CALL_EVENT_ANSWERED, call->peer_uri);
 
 	if (msg_ctype_cmp(&msg->ctyp, "multipart", "mixed"))
 		(void)sdp_decode_multipart(&msg->ctyp.params, msg->mb);
@@ -1510,26 +1580,6 @@ static void sipsess_estab_handler(const struct sip_msg *msg, void *arg)
 }
 
 
-static void call_handle_info_req(struct call *call, const struct sip_msg *req)
-{
-	struct pl body;
-	bool pfu;
-	int err;
-
-	pl_set_mbuf(&body, req->mb);
-
-	err = mctrl_handle_media_control(&body, &pfu);
-	if (err)
-		return;
-
-	debug("call: receive media control: fast_update=%d\n", pfu);
-
-	if (pfu) {
-		video_update_picture(call->video);
-	}
-}
-
-
 static void dtmfend_handler(void *arg)
 {
 	struct call *call = arg;
@@ -1539,6 +1589,19 @@ static void dtmfend_handler(void *arg)
 }
 
 
+static void sipsess_send_info_handler(int err, const struct sip_msg *msg,
+				void *arg)
+{
+	(void)arg;
+
+	if (err)
+		warning("call: sending DTMF INFO failed (%m)", err);
+	else if (msg && msg->scode != 200)
+		warning("call: sending DTMF INFO failed (scode: %d)",
+				msg->scode);
+}
+
+
 static void sipsess_info_handler(struct sip *sip, const struct sip_msg *msg,
 				 void *arg)
 {
@@ -1561,8 +1624,8 @@ static void sipsess_info_handler(struct sip *sip, const struct sip_msg *msg,
 			char s = toupper(sig.p[0]);
 			uint32_t duration = pl_u32(&dur);
 
-			info("call: received DTMF: '%c' (duration=%r)\n",
-			     s, &dur);
+			info("call: received SIP INFO DTMF: '%c' "
+			     "(duration=%r)\n", s, &dur);
 
 			(void)sip_reply(sip, msg, 200, "OK");
 
@@ -1573,11 +1636,6 @@ static void sipsess_info_handler(struct sip *sip, const struct sip_msg *msg,
 			}
 		}
 	}
-	else if (msg_ctype_cmp(&msg->ctyp,
-			       "application", "media_control+xml")) {
-		call_handle_info_req(call, msg);
-		(void)sip_reply(sip, msg, 200, "OK");
-	}
 	else {
 		(void)sip_reply(sip, msg, 488, "Not Acceptable Here");
 	}
@@ -1801,6 +1859,14 @@ int call_accept(struct call *call, struct sipsess_sock *sess_sock,
 }
 
 
+static void delayed_answer_handler(void *arg)
+{
+	struct call *call = arg;
+
+	(void)call_answer(call, 200, VIDMODE_ON);
+}
+
+
 static void sipsess_progr_handler(const struct sip_msg *msg, void *arg)
 {
 	struct call *call = arg;
@@ -1918,6 +1984,35 @@ static int send_invite(struct call *call)
 }
 
 
+static int send_dtmf_info(struct call *call, char key)
+{
+	struct mbuf *body;
+	int err;
+
+	if ((key < '0' || key > '9') &&
+	    (key < 'a' || key > 'd') &&
+	    (key != '*') &&
+	    (key != '#'))
+		return EINVAL;
+
+	body = mbuf_alloc(25);
+	mbuf_printf(body, "Signal=%c\r\nDuration=250\r\n", key);
+	mbuf_set_pos(body, 0);
+
+	err = sipsess_info(call->sess, "application/dtmf-relay", body,
+			   sipsess_send_info_handler, call);
+	if (err) {
+		warning("call: sipsess_info for DTMF failed (%m)\n", err);
+		goto out;
+	}
+
+ out:
+	mem_deref(body);
+
+	return err;
+}
+
+
 /**
  * Get the current call duration in seconds
  *
@@ -2290,6 +2385,19 @@ uint32_t call_linenum(const struct call *call)
 }
 
 
+/**
+ * Get the answer delay of this call
+ *
+ * @param call Call object
+ *
+ * @return answer delay in ms
+ */
+int32_t call_answer_delay(const struct call *call)
+{
+	return call ? call->adelay : -1;
+}
+
+
 /**
  * Find the call with a given line number
  *
@@ -2350,3 +2458,41 @@ void call_set_current(struct list *calls, struct call *call)
 	list_unlink(&call->le);
 	list_append(calls, &call->le, call);
 }
+
+
+/**
+ * Set stream sdp media line direction attribute
+ *
+ * @param call Call object
+ * @param a    Audio SDP direction
+ * @param v    Video SDP direction if video available
+ *
+ * @return int	0 if success, errorcode otherwise
+ */
+int call_set_media_direction(struct call *call, enum sdp_dir a, enum sdp_dir v)
+{
+	if (!call)
+		return EINVAL;
+
+	stream_set_ldir(audio_strm(call_audio(call)), a);
+
+	if (video_strm(call_video(call))) {
+		if (vidisp_find(baresip_vidispl(), NULL) == NULL)
+			stream_set_ldir(video_strm(
+				call_video(call)), v & SDP_SENDONLY);
+		else
+			stream_set_ldir(video_strm(call_video(call)), v);
+
+	}
+
+	return 0;
+}
+
+
+void call_start_answtmr(struct call *call, uint32_t ms)
+{
+	if (!call)
+		return;
+
+	tmr_start(&call->tmr_answ, ms, delayed_answer_handler, call);
+}
