@@ -1,7 +1,7 @@
 /**
  * @file core.h  Internal API
  *
- * Copyright (C) 2010 Creytiv.com
+ * Copyright (C) 2010 Alfred E. Heggestad
  */
 
 
@@ -18,17 +18,6 @@
 #endif
 
 
-/**
- * RFC 3551:
- *
- *    0 -  95  Static payload types
- *   96 - 127  Dynamic payload types
- */
-enum {
-	PT_CN       = 13,
-};
-
-
 /** Media constants */
 enum {
 	AUDIO_BANDWIDTH = 128000,  /**< Bandwidth for audio in bits/s      */
@@ -53,7 +42,10 @@ struct account {
 	char *aor;                   /**< Local SIP uri                      */
 
 	/* parameters: */
+	bool sipans;                 /**< Allow SIP header auto answer mode  */
 	enum answermode answermode;  /**< Answermode for incoming calls      */
+	uint32_t adelay;             /**< Delay for delayed auto answer [ms] */
+	enum dtmfmode dtmfmode;      /**< Send type for DTMF tones           */
 	struct le acv[16];           /**< List elements for aucodecl         */
 	struct list aucodecl;        /**< List of preferred audio-codecs     */
 	char *auth_user;             /**< Authentication username            */
@@ -65,8 +57,10 @@ struct account {
 	char *outboundv[2];          /**< Optional SIP outbound proxies      */
 	uint32_t ptime;              /**< Configured packet time in [ms]     */
 	uint32_t regint;             /**< Registration interval in [seconds] */
+	uint32_t fbregint;           /**< Fallback R. interval in [seconds]  */
 	uint32_t rwait;              /**< R. Int. in [%] from proxy expiry   */
 	uint32_t pubint;             /**< Publication interval in [seconds]  */
+	uint32_t prio;               /**< Prio for serial registration       */
 	char *regq;                  /**< Registration Q-value               */
 	char *sipnat;                /**< SIP Nat mechanism                  */
 	char *stun_user;             /**< STUN Username                      */
@@ -84,24 +78,6 @@ struct account {
 };
 
 
-/*
- * Audio Player
- */
-
-struct auplay_st {
-	struct auplay *ap;
-};
-
-
-/*
- * Audio Source
- */
-
-struct ausrc_st {
-	const struct ausrc *as;
-};
-
-
 /*
  * Audio Stream
  */
@@ -146,10 +122,8 @@ int  call_info(struct re_printf *pf, const struct call *call);
 int  call_reset_transp(struct call *call, const struct sa *laddr);
 int  call_af(const struct call *call);
 void call_set_xrtpstat(struct call *call);
-struct account *call_account(const struct call *call);
 void call_set_custom_hdrs(struct call *call, const struct list *hdrs);
 
-
 /*
 * Custom headers
 */
@@ -167,13 +141,6 @@ int conf_get_csv(const struct conf *conf, const char *name,
 int conf_get_float(const struct conf *conf, const char *name, double *val);
 
 
-/*
- * Media control
- */
-
-int mctrl_handle_media_control(struct pl *body, bool *pfu);
-
-
 /*
  * Metric
  */
@@ -220,6 +187,7 @@ int  reg_register(struct reg *reg, const char *reg_uri,
 		    const char *params, uint32_t regint, const char *outbound);
 void reg_unregister(struct reg *reg);
 bool reg_isok(const struct reg *reg);
+bool reg_failed(const struct reg *reg);
 int  reg_debug(struct re_printf *pf, const struct reg *reg);
 int  reg_json_api(struct odict *od, const struct reg *reg);
 int  reg_status(struct re_printf *pf, const struct reg *reg);
@@ -286,6 +254,7 @@ enum {STREAM_PRESZ = 4+12}; /* same as RTP_HEADER_SIZE */
 typedef void (stream_rtp_h)(const struct rtp_header *hdr,
 			    struct rtpext *extv, size_t extc,
 			    struct mbuf *mb, unsigned lostc, void *arg);
+typedef int (stream_pt_h)(uint8_t pt, struct mbuf *mb, void *arg);
 
 
 /** Defines a generic media stream */
@@ -296,6 +265,7 @@ struct stream {
 	struct le le;            /**< Linked list element                   */
 	struct config_avt cfg;   /**< Stream configuration                  */
 	struct sdp_media *sdp;   /**< SDP Media line                        */
+	enum sdp_dir ldir;       /**< SDP direction of the stream           */
 	struct rtp_sock *rtp;    /**< RTP Socket                            */
 	struct rtcp_stats rtcp_stats;/**< RTCP statistics                   */
 	struct jbuf *jbuf;       /**< Jitter Buffer for incoming RTP        */
@@ -314,8 +284,10 @@ struct stream {
 	uint32_t pseq;           /**< Sequence number for incoming RTP      */
 	bool pseq_set;           /**< True if sequence number is set        */
 	int pt_enc;              /**< Payload type for encoding             */
+	int pt_dec;              /**< Payload type for decoding             */
 	bool rtcp_mux;           /**< RTP/RTCP multiplex supported by peer  */
 	bool jbuf_started;       /**< True if jitter-buffer was started     */
+	stream_pt_h *pth;        /**< Stream payload type handler           */
 	struct tmr tmr_rtp;      /**< Timer for detecting RTP timeout       */
 	uint64_t ts_last;        /**< Timestamp of last received RTP pkt    */
 	bool terminated;         /**< Stream is terminated flag             */
@@ -342,7 +314,8 @@ int  stream_alloc(struct stream **sp, struct list *streaml,
 		  const struct mnat *mnat, struct mnat_sess *mnat_sess,
 		  const struct menc *menc, struct menc_sess *menc_sess,
 		  bool offerer,
-		  stream_rtp_h *rtph, stream_rtcp_h *rtcph, void *arg);
+		  stream_rtp_h *rtph, stream_rtcp_h *rtcph, stream_pt_h *pth,
+		  void *arg);
 int  stream_send(struct stream *s, bool ext, bool marker, int pt, uint32_t ts,
 		 struct mbuf *mb);
 void stream_update_encoder(struct stream *s, int pt_enc);
@@ -355,9 +328,9 @@ void stream_reset(struct stream *s);
 void stream_set_bw(struct stream *s, uint32_t bps);
 int  stream_print(struct re_printf *pf, const struct stream *s);
 void stream_enable_rtp_timeout(struct stream *strm, uint32_t timeout_ms);
-int  stream_jbuf_reset(struct stream *strm,
-		       uint32_t frames_min, uint32_t frames_max);
 bool stream_is_ready(const struct stream *strm);
+int  stream_decode(struct stream *s);
+void stream_silence_on(struct stream *s, bool on);
 
 
 /*
@@ -377,8 +350,6 @@ int ua_print_allowed(struct re_printf *pf, const struct ua *ua);
 
 struct video;
 
-
-bool video_is_started(const struct video *v);
 int  video_decoder_set(struct video *v, struct vidcodec *vc, int pt_rx,
 		       const char *fmtp);
 void video_update_picture(struct video *v);
