@@ -1,7 +1,7 @@
 /**
  * @file src/h264.c  H.264 video codec packetization (RFC 3984)
  *
- * Copyright (C) 2010 - 2015 Creytiv.com
+ * Copyright (C) 2010 - 2015 Alfred E. Heggestad
  */
 #include <string.h>
 #include <re.h>
@@ -9,33 +9,6 @@
 #include <baresip.h>
 
 
-int h264_hdr_encode(const struct h264_hdr *hdr, struct mbuf *mb)
-{
-	uint8_t v;
-
-	v = hdr->f<<7 | hdr->nri<<5 | hdr->type<<0;
-
-	return mbuf_write_u8(mb, v);
-}
-
-
-int h264_hdr_decode(struct h264_hdr *hdr, struct mbuf *mb)
-{
-	uint8_t v;
-
-	if (mbuf_get_left(mb) < 1)
-		return ENOENT;
-
-	v = mbuf_read_u8(mb);
-
-	hdr->f    = v>>7 & 0x1;
-	hdr->nri  = v>>5 & 0x3;
-	hdr->type = v>>0 & 0x1f;
-
-	return 0;
-}
-
-
 int h264_fu_hdr_encode(const struct h264_fu *fu, struct mbuf *mb)
 {
 	uint8_t v = fu->s<<7 | fu->s<<6 | fu->r<<5 | fu->type;
@@ -129,7 +102,7 @@ int h264_nal_send(bool first, bool last,
 		const uint8_t nri  = hdr & 0x60;
 		const size_t sz = maxsz - 2;
 
-		fu_hdr[0] = nri | H264_NAL_FU_A;
+		fu_hdr[0] = nri | H264_NALU_FU_A;
 		fu_hdr[1] = first ? (1<<7 | type) : type;
 
 		while (size > sz) {
@@ -153,6 +126,19 @@ int h264_nal_send(bool first, bool last,
 }
 
 
+/**
+ * Packetize an H.264 bitstream with one or more NAL units
+ * with Annex-B startcode (3-byte or 4-byte startcode)
+ *
+ * @param rtp_ts  RTP timestamp
+ * @param buf     Input buffer
+ * @param len     Buffer length
+ * @param pktsize Maximum RTP packet size
+ * @param pkth    Packet handler
+ * @param arg     Handler argument
+ *
+ * @return 0 if success, otherwise errorcode
+ */
 int h264_packetize(uint64_t rtp_ts, const uint8_t *buf, size_t len,
 		   size_t pktsize, videnc_packet_h *pkth, void *arg)
 {
@@ -182,33 +168,7 @@ int h264_packetize(uint64_t rtp_ts, const uint8_t *buf, size_t len,
 }
 
 
-/**
- * Get the name of an H.264 nal unit
- *
- * @param type NAL unit type
- *
- * @return A string containing the NAL unit name
- */
-const char *h264_nalunit_name(int type)
+bool h264_is_keyframe(int type)
 {
-	switch (type) {
-
-	case H264_NAL_SLICE:       return "SLICE";
-	case H264_NAL_DPA:         return "DPA";
-	case H264_NAL_DPB:         return "DPB";
-	case H264_NAL_DPC:         return "DPC";
-	case H264_NAL_IDR_SLICE:   return "IDR_SLICE";
-	case H264_NAL_SEI:         return "SEI";
-	case H264_NAL_SPS:         return "SPS";
-	case H264_NAL_PPS:         return "PPS";
-	case H264_NAL_AUD:         return "AUD";
-	case H264_NAL_FILLER_DATA: return "FILLER";
-
-	case H264_NAL_STAP_A:      return "STAP-A";
-	case H264_NAL_STAP_B:      return "STAP-B";
-	case H264_NAL_FU_A:        return "FU-A";
-	case H264_NAL_FU_B:        return "FU-B";
-	}
-
-	return "???";
+	return type == H264_NALU_IDR_SLICE;
 }
