@@ -1,7 +1,7 @@
 /**
  * @file src/main.c  Main application code
  *
- * Copyright (C) 2010 - 2015 Creytiv.com
+ * Copyright (C) 2010 - 2021 Alfred E. Heggestad
  */
 #ifdef SOLARIS
 #define __EXTENSIONS__ 1
@@ -10,7 +10,9 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#ifdef HAVE_GETOPT
 #include <getopt.h>
+#endif
 #include <re.h>
 #include <baresip.h>
 
@@ -20,6 +22,7 @@ static void signal_handler(int sig)
 	static bool term = false;
 
 	if (term) {
+		module_app_unload();
 		mod_close();
 		exit(0);
 	}
@@ -107,7 +110,7 @@ int main(int argc, char *argv[])
 	setbuf(stdout, NULL);
 
 	(void)re_fprintf(stdout, "baresip v%s"
-			 " Copyright (C) 2010 - 2020"
+			 " Copyright (C) 2010 - 2021"
 			 " Alfred E. Heggestad et al.\n",
 			 BARESIP_VERSION);
 
@@ -119,6 +122,7 @@ int main(int argc, char *argv[])
 
 	tmr_init(&tmr_quit);
 
+#ifdef HAVE_GETOPT
 	for (;;) {
 		const int c = getopt(argc, argv, "46de:f:p:hu:n:vst:m:");
 		if (0 > c)
@@ -197,6 +201,10 @@ int main(int argc, char *argv[])
 			break;
 		}
 	}
+#else
+	(void)argc;
+	(void)argv;
+#endif
 
 	err = conf_configure();
 	if (err) {
