@@ -1,7 +1,7 @@
 /**
  * @file src/net.c Networking code
  *
- * Copyright (C) 2010 - 2016 Creytiv.com
+ * Copyright (C) 2010 - 2016 Alfred E. Heggestad
  */
 #include <re.h>
 #include <baresip.h>
@@ -170,7 +170,7 @@ static int net_dns_srv_get(const struct network *net,
 	if (net->nsnf) {
 		offset = *n;
 		if ((offset + net->nsnf) > limit) {
-			warning("net: too many DNS nameservers, "
+			debug("net: too many DNS nameservers, "
 					"fallback DNS ignored\n");
 			return 0;
 		}
@@ -189,7 +189,7 @@ static int net_dns_srv_get(const struct network *net,
 /*
  * Check for DNS Server updates
  */
-static void dns_refresh(struct network *net)
+void net_dns_refresh(struct network *net)
 {
 	struct sa nsv[NET_MAX_NS];
 	uint32_t nsn;
@@ -215,7 +215,7 @@ static void ipchange_handler(void *arg)
 
 	tmr_start(&net->tmr, net->interval * 1000, ipchange_handler, net);
 
-	dns_refresh(net);
+	net_dns_refresh(net);
 
 	change = net_check(net);
 	if (change && net->ch) {
@@ -546,7 +546,7 @@ int net_use_nameserver(struct network *net, const struct sa *srvv, size_t srvc)
 		}
 	}
 
-	dns_refresh(net);
+	net_dns_refresh(net);
 
 	return 0;
 }
@@ -728,22 +728,6 @@ struct dnsc *net_dnsc(const struct network *net)
 }
 
 
-/**
- * Get the network domain name
- *
- * @param net Network instance
- *
- * @return Network domain
- */
-const char *net_domain(const struct network *net)
-{
-	if (!net)
-		return NULL;
-
-	return net->domain[0] ? net->domain : NULL;
-}
-
-
 /**
  * Print networking debug information
  *
@@ -769,7 +753,6 @@ int net_debug(struct re_printf *pf, const struct network *net)
 			  net_af_enabled(net, AF_INET6) ? "E" : ".",
 			  print_addr, &net->laddr6);
 #endif
-	err |= re_hprintf(pf, " Domain: %s\n", net->domain);
 
 	err |= re_hprintf(pf, "net interfaces:\n");
 	err |= net_if_apply(if_debug_handler, argv);
