@@ -1,7 +1,7 @@
 /**
  * @file reg.c  Register Client
  *
- * Copyright (C) 2010 Creytiv.com
+ * Copyright (C) 2010 Alfred E. Heggestad
  */
 #include <re.h>
 #include <baresip.h>
@@ -14,6 +14,7 @@ struct reg {
 	struct ua *ua;               /**< Pointer to parent UA object        */
 	struct sipreg *sipreg;       /**< SIP Register client                */
 	int id;                      /**< Registration ID (for SIP outbound) */
+	int regint;                  /**< Registration interval              */
 
 	/* status: */
 	uint16_t scode;              /**< Registration status code           */
@@ -48,6 +49,8 @@ static int sipmsg_af(const struct sip_msg *msg)
 
 	case SIP_TRANSP_TCP:
 	case SIP_TRANSP_TLS:
+	case SIP_TRANSP_WS:
+	case SIP_TRANSP_WSS:
 		err = tcp_conn_local_get(sip_msg_tcpconn(msg), &laddr);
 		break;
 
@@ -96,14 +99,22 @@ static bool contact_handler(const struct sip_hdr *hdr,
 static void register_handler(int err, const struct sip_msg *msg, void *arg)
 {
 	struct reg *reg = arg;
+	const struct account *acc = ua_account(reg->ua);
 	const struct sip_hdr *hdr;
+	uint32_t prio = account_prio(acc);
+	enum ua_event evok =  reg->regint ?
+		UA_EVENT_REGISTER_OK : UA_EVENT_FALLBACK_OK;
+	enum ua_event evfail = reg->regint ?
+		UA_EVENT_REGISTER_FAIL : UA_EVENT_FALLBACK_FAIL;
 
 	if (err) {
-		warning("reg: %s: Register: %m\n", ua_aor(reg->ua), err);
+		if (reg->regint)
+			warning("reg: %s (prio %u): Register: %m\n",
+				account_aor(acc), prio, err);
 
 		reg->scode = 999;
 
-		ua_event(reg->ua, UA_EVENT_REGISTER_FAIL, NULL, "%m", err);
+		ua_event(reg->ua, evfail, NULL, "%m", err);
 		return;
 	}
 
@@ -120,10 +131,10 @@ static void register_handler(int err, const struct sip_msg *msg, void *arg)
 		n_bindings = sip_msg_hdr_count(msg, SIP_HDR_CONTACT);
 		reg->af    = sipmsg_af(msg);
 
-		if (msg->scode != reg->scode) {
-			ua_printf(reg->ua, "{%d/%s/%s} %u %r (%s)"
+		if (msg->scode != reg->scode && reg->regint) {
+			ua_printf(reg->ua, "(prio %u) {%d/%s/%s} %u %r (%s)"
 				  " [%u binding%s]\n",
-				  reg->id, sip_transp_name(msg->tp),
+				  prio, reg->id, sip_transp_name(msg->tp),
 				  af_name(reg->af), msg->scode, &msg->reason,
 				  reg->srv, n_bindings,
 				  1==n_bindings?"":"s");
@@ -144,17 +155,17 @@ static void register_handler(int err, const struct sip_msg *msg, void *arg)
 			}
 		}
 
-		ua_event(reg->ua, UA_EVENT_REGISTER_OK, NULL, "%u %r",
+		ua_event(reg->ua, evok, NULL, "%u %r",
 			 msg->scode, &msg->reason);
 	}
 	else if (msg->scode >= 300) {
 
-		warning("reg: %s: %u %r (%s)\n", ua_aor(reg->ua),
-			msg->scode, &msg->reason, reg->srv);
+		warning("reg: %s (prio %u): %u %r (%s)\n", account_aor(acc),
+				prio, msg->scode, &msg->reason, reg->srv);
 
 		reg->scode = msg->scode;
 
-		ua_event(reg->ua, UA_EVENT_REGISTER_FAIL, NULL, "%u %r",
+		ua_event(reg->ua, evfail, NULL, "%u %r",
 			 msg->scode, &msg->reason);
 	}
 }
@@ -186,14 +197,17 @@ int reg_register(struct reg *reg, const char *reg_uri, const char *params,
 	struct account *acc;
 	const char *routev[1];
 	int err;
+	bool failed;
 
 	if (!reg || !reg_uri)
 		return EINVAL;
 
 	reg->scode = 0;
+	reg->regint = regint;
 	routev[0] = outbound;
 	acc = ua_account(reg->ua);
 
+	failed = sipreg_failed(reg->sipreg);
 	reg->sipreg = mem_deref(reg->sipreg);
 	err = sipreg_register(&reg->sipreg, uag_sip(), reg_uri,
 			      account_aor(acc),
@@ -209,9 +223,15 @@ int reg_register(struct reg *reg, const char *reg_uri, const char *params,
 	if (err)
 		return err;
 
-	if (acc->rwait)
+	if (acc && acc->rwait)
 		err = sipreg_set_rwait(reg->sipreg, acc->rwait);
 
+	if (acc && acc->fbregint)
+		err = sipreg_set_fbregint(reg->sipreg, acc->fbregint);
+
+	if (failed)
+		sipreg_incfailc(reg->sipreg);
+
 	return err;
 }
 
@@ -230,10 +250,19 @@ void reg_unregister(struct reg *reg)
 
 bool reg_isok(const struct reg *reg)
 {
-	if (!reg)
+	if (!reg || !reg->sipreg)
 		return false;
 
-	return 200 <= reg->scode && reg->scode <= 299;
+	return sipreg_registered(reg->sipreg);
+}
+
+
+bool reg_failed(const struct reg *reg)
+{
+	if (!reg || !reg->sipreg)
+		return false;
+
+	return sipreg_failed(reg->sipreg);
 }
 
 
@@ -256,14 +285,19 @@ static const char *print_scode(uint16_t scode)
 int reg_debug(struct re_printf *pf, const struct reg *reg)
 {
 	int err = 0;
+	bool fb;
 
 	if (!reg)
 		return 0;
 
+	fb = !sipreg_proxy_expires(reg->sipreg) && reg->scode;
+
 	err |= re_hprintf(pf, "\nRegister client:\n");
 	err |= re_hprintf(pf, " id:     %d\n", reg->id);
-	err |= re_hprintf(pf, " scode:  %u (%s)\n",
-			  reg->scode, print_scode(reg->scode));
+	err |= re_hprintf(pf, " scode:  %u (%s%s)\n",
+			  reg->scode,
+			   fb ? "fallback " : "",
+			  print_scode(reg->scode));
 	err |= re_hprintf(pf, " srv:    %s\n", reg->srv);
 	err |= re_hprintf(pf, " af:     %s\n", af_name(reg->af));
 
@@ -288,6 +322,8 @@ int reg_json_api(struct odict *od, const struct reg *reg)
 
 	err |= odict_entry_add(od, "id", ODICT_INT, (int64_t) reg->id);
 	err |= odict_entry_add(od, "state", ODICT_BOOL, reg_isok(reg));
+	err |= odict_entry_add(od, "expires", ODICT_INT,
+			       (int64_t) sipreg_proxy_expires(reg->sipreg));
 	err |= odict_entry_add(od, "code", ODICT_INT, (int64_t) reg->scode);
 	if (reg->srv)
 		err |= odict_entry_add(od, "srv", ODICT_STRING, reg->srv);
@@ -301,11 +337,24 @@ int reg_json_api(struct odict *od, const struct reg *reg)
 
 int reg_status(struct re_printf *pf, const struct reg *reg)
 {
+	uint32_t pexpires;
+	bool fb;
+
 	if (!reg)
 		return 0;
 
-	return re_hprintf(pf, " %s %s Expires %us", print_scode(reg->scode),
-			reg->srv, sipreg_proxy_expires(reg->sipreg));
+	pexpires = sipreg_proxy_expires(reg->sipreg);
+	fb = !pexpires && reg->scode;
+
+	if (pexpires) {
+		return re_hprintf(pf, " %s %s Expires %us",
+				print_scode(reg->scode), reg->srv, pexpires);
+	}
+	else {
+		return re_hprintf(pf, " %s%s %s",
+				fb ? "FB-" : "   ",
+				print_scode(reg->scode), reg->srv);
+	}
 }
 
 
