@@ -1,7 +1,7 @@
 /**
  * @file vidsrc.c Video Source
  *
- * Copyright (C) 2010 Creytiv.com
+ * Copyright (C) 2010 Alfred E. Heggestad
  */
 
 #include <re.h>
@@ -9,12 +9,6 @@
 #include "core.h"
 
 
-/** Video Source state */
-struct vidsrc_st {
-	struct vidsrc *vs;  /**< Video Source */
-};
-
-
 static void destructor(void *arg)
 {
 	struct vidsrc *vs = arg;
@@ -118,16 +112,3 @@ int vidsrc_alloc(struct vidsrc_st **stp, struct list *vidsrcl,
 	return vs->alloch(stp, vs, ctx, prm, size, fmt, dev,
 			  frameh, errorh, arg);
 }
-
-
-/**
- * Get the video source module from a video source state
- *
- * @param st Video source state
- *
- * @return Video source module
- */
-struct vidsrc *vidsrc_get(struct vidsrc_st *st)
-{
-	return st ? st->vs : NULL;
-}
