@@ -1,7 +1,7 @@
 /**
  * @file test/account.c  Tests for account
  *
- * Copyright (C) 2010 - 2017 Creytiv.com
+ * Copyright (C) 2010 - 2017 Alfred E. Heggestad
  */
 #include <string.h>
 #include <re.h>
@@ -30,7 +30,6 @@ static const char str[] =
 	";mwi=no"
 	";call_transfer=no"
 	";audio_source=null,null"
-	";audio_codecs=RAW-CODEC"
 	;
 
 
@@ -38,12 +37,8 @@ int test_account(void)
 {
 	struct account *acc = NULL;
 	struct sip_addr *addr;
-	const struct aucodec *ac;
-	struct list *lst;
 	int err = 0;
 
-	mock_aucodec_register(baresip_aucodecl());
-
 	err = account_alloc(&acc, str);
 	TEST_ERR(err);
 	ASSERT_TRUE(acc != NULL);
@@ -76,14 +71,7 @@ int test_account(void)
 	ASSERT_STREQ("no", account_mwi(acc));
 	ASSERT_STREQ("no", account_call_transfer(acc));
 
-	lst = account_aucodecl(acc);
-	ac = list_ledata(list_head(lst));
-	ASSERT_EQ(1, list_count(lst));
-	ASSERT_STREQ("RAW-CODEC", ac->name);
-
  out:
-	mock_aucodec_unregister();
-
 	mem_deref(acc);
 	return err;
 }
