@@ -1,7 +1,7 @@
 /**
  * @file test/call.c  Baresip selftest -- call
  *
- * Copyright (C) 2010 - 2015 Creytiv.com
+ * Copyright (C) 2010 - 2015 Alfred E. Heggestad
  */
 #include <string.h>
 #include <stdlib.h>
@@ -152,7 +152,7 @@ static void event_handler(struct ua *ua, enum ua_event ev,
 
 #if 1
 	info("test: [ %s ] event: %s (%s)\n",
-		  ua_aor(ua), uag_event_str(ev), prm);
+	     account_aor(ua_account(ua)), uag_event_str(ev), prm);
 #endif
 
 	ASSERT_TRUE(f != NULL);
@@ -640,8 +640,10 @@ int test_call_max(void)
 	unsigned i;
 	int err = 0;
 
-	/* Set the max-calls limit */
-	conf_config()->call.max_calls = 1;
+	/* Set the max-calls limit to accept 1 incoming call. */
+	/* We start 2 calls from a.ua to b.ua. */
+	/* This are 2 outgoing calls and 1 incoming. */
+	conf_config()->call.max_calls = 3;
 
 	fixture_init(f);
 
@@ -1208,6 +1210,50 @@ int test_call_tcp(void)
 }
 
 
+int test_call_deny_udp(void)
+{
+	struct fixture fix, *f = &fix;
+	int err = 0;
+	char curi[256];
+
+	fixture_init(f);
+
+	mem_deref(f->a.ua);
+	mem_deref(f->b.ua);
+	err = ua_alloc(&f->a.ua, "A <sip:a@127.0.0.1;transport=tcp>;regint=0");
+	TEST_ERR(err);
+	err = ua_alloc(&f->b.ua, "B <sip:b@127.0.0.1;transport=tcp>;regint=0");
+	TEST_ERR(err);
+
+	f->a.peer = &f->b;
+	f->b.peer = &f->a;
+
+	f->b.n_closed = 1;
+	f->estab_action = ACTION_RECANCEL;
+
+	/* Make a call using UDP-transport */
+	re_snprintf(curi, sizeof(curi), "sip:b@%J;transport=udp",
+			&f->laddr_udp);
+	err = ua_connect(f->a.ua, 0, NULL, curi, VIDMODE_OFF);
+	TEST_ERR(err);
+
+	/* run main-loop with timeout, wait for events */
+	err = re_main_timeout(5000);
+	TEST_ERR(err);
+	TEST_ERR(fix.err);
+
+	ASSERT_EQ(0, fix.a.n_established);
+	ASSERT_EQ(0, fix.b.n_established);
+	ASSERT_EQ(1, fix.a.n_closed);
+	ASSERT_EQ(0, fix.b.n_incoming);
+
+ out:
+	fixture_close(f);
+
+	return err;
+}
+
+
 /*
  *  Step 1. Call from A to B
  *
