@@ -1,7 +1,7 @@
 /**
  * @file test/message.c  Baresip selftest -- message sending
  *
- * Copyright (C) 2010 Creytiv.com
+ * Copyright (C) 2010 Alfred E. Heggestad
  */
 #include <string.h>
 #include <re.h>
@@ -136,8 +136,8 @@ static int endpoint_alloc(struct endpoint **epp, struct test *test,
 	ep->test = test;
 
 	if (re_snprintf(aor, sizeof(aor),
-			"%s <sip:%s@127.0.0.1;transport=%s>;regint=0",
-			name, name,
+			"%s <sip:%s@%j;transport=%s>;regint=0",
+			name, name, &laddr,
 			sip_transp_name(transp)) < 0) {
 		err = ENOMEM;
 		goto out;
