@@ -1,7 +1,7 @@
 /**
  * @file test.h  Selftest for Baresip core -- internal API
  *
- * Copyright (C) 2010 Creytiv.com
+ * Copyright (C) 2010 Alfred E. Heggestad
  */
 
 
@@ -117,16 +117,12 @@ struct dns_server {
 int dns_server_alloc(struct dns_server **srvp, bool rotate);
 int dns_server_add_a(struct dns_server *srv,
 		     const char *name, uint32_t addr);
+int dns_server_add_aaaa(struct dns_server *srv, const char *name,
+			const uint8_t *addr);
 int dns_server_add_srv(struct dns_server *srv, const char *name,
 		       uint16_t pri, uint16_t weight, uint16_t port,
 		       const char *target);
 
-/*
- * Mock Audio-codec
- */
-
-void mock_aucodec_register(struct list *aucodecl);
-void mock_aucodec_unregister(void);
 
 /*
  * Mock Audio-source
@@ -226,6 +222,7 @@ int test_call_reject(void);
 int test_call_rtcp(void);
 int test_call_rtp_timeout(void);
 int test_call_tcp(void);
+int test_call_deny_udp(void);
 int test_call_transfer(void);
 int test_call_video(void);
 int test_call_webrtc(void);
@@ -233,6 +230,7 @@ int test_cmd(void);
 int test_cmd_long(void);
 int test_contact(void);
 int test_event(void);
+int test_h264(void);
 int test_message(void);
 int test_network(void);
 int test_play(void);
